% ICIP 2013:
% << Feature Normalization for Part-Based Image Classification >>
% by Lingxi Xie, Qi Tian and Bo Zhang.
% ---------------------------------------------------------------------------
% Program Name:         Classifier_SEP
% Program Description:  Separate feature normalization program with adjustable parameters: power and coefficient.
% ---------------------------------------------------------------------------
% File Name:            FeatureNormalization.m
% File Description:     The feature normalization function.
% ---------------------------------------------------------------------------
% Code by Lingxi Xie, Sep, 2013.

function finished = FeatureNormalization(featurePathO, featurePath, row, binSize, power, coef)

% Loading the structure of feature set.
[feature, going] = LoadFeatures(featurePathO, false);
if (going)
    if (~isdir(featurePath))
        mkdir(featurePath);
    end
% Enumerating on all the categories.
    for i = 1: 1: feature.categories
        categoryName = feature.cate(i).name;
        images = feature.cate(i).images;
        fprintf('Feature normalization of category # %03d: %s, begins.\r\n', i, categoryName);
% The original feature directory.
        featureDirectoryO = fullfile(featurePathO, categoryName);
% The normalized feature directory.
        featureDirectory = fullfile(featurePath, categoryName);
        if (~isdir(featureDirectory))
            mkdir(featureDirectory);
        end
        for j = 1: 1: images
            [~, name_] = fileparts(feature.cate(i).image(j).name);
% The original feature file.
            featureFileO = fullfile(featureDirectoryO, [name_, '.mat']);
% The normalized feature file.
            featureFile = fullfile(featureDirectory, [name_, '.mat']);
            if (~exist(featureFile, 'file'))
                features = load(featureFileO);
                features = features.features;
                dimension = length(features);
                bins = dimension / binSize;
% Dividing the feature into bins according to the pre-defined structure.
                if (row)
                    features = reshape(features, binSize, bins);
                    features = features';
                else
                    features = reshape(features, bins, binSize);
                end
                for p = 1: 1: bins
% When power is -1, it means L_inf normalization (maximization).
                    if (power == -1)
                        s = max(abs(features(p, :)));
                    else
                        s = (sum(abs(features(p, :)) .^ power)) .^ (1 / power);
                    end
                    if (s ~= 0)
                        features(p, :) = features(p, :) / s;
                    end
                end
                features = features(:)';
% The normalization coefficient.
                features = features * coef; %#ok<NASGU>
% Saving the normalized feature.
                save(featureFile, 'features');
                fprintf('    Image #%03d out of %d images of category %03d is finished.\r\n', ...
                    j, images, i);
            else
                fprintf('    Image #%03d out of %d images of category %03d existed.\r\n', ...
                    j, images, i);
            end
        end
        fprintf('Feature normalization of category # %03d: %s, is finished.\r\n', i, categoryName);
    end
    finished = true;
else
    finished = false;
end

end
