% ICIP 2013:
% << Feature Normalization for Part-Based Image Classification >>
% by Lingxi Xie, Qi Tian and Bo Zhang.
% ---------------------------------------------------------------------------
% Program Name:         Classifier_SEP
% Program Description:  Separate feature normalization program with adjustable parameters: power and coefficient.
% ---------------------------------------------------------------------------
% File Name:            LoadFeatures.m
% File Description:     Loading feature vectors for classification.
% ---------------------------------------------------------------------------
% Code by Lingxi Xie, Sep, 2013.

function [feature, finished] = LoadFeatures(featurePath, loadVector)

% Initialization of feature structure.
feature.categories = 0;
feature.images = 0;

% Checking the existence of feature path.
if (exist(featurePath, 'dir'))
    fprintf('Begin loading features of all images.\n');
% Finding all the categories.
    categoryDirectories = dir(featurePath);
    for i = 1: 1: length(categoryDirectories)
        categoryName = categoryDirectories(i).name;
% Skipping the system directory.
        if (~strcmp(categoryName, '.') && ~strcmp(categoryName, '..'))
            feature.categories = feature.categories + 1;
            ca = feature.categories;
            feature.cate(ca).images = 0;
            feature.cate(ca).name = categoryName;
% The feature directory.
            featureDirectory = fullfile(featurePath, categoryName);
% Finding all the feature files with a '.mat' suffix.
% If you use other suffices, change the setting here.
            featureFiles = dir(fullfile(featureDirectory, '*.mat'));
            count = length(featureFiles);
            for j = 1: 1: count
                feature.cate(ca).images = feature.cate(ca).images + 1;
                im = feature.cate(ca).images;
                [~, name_] = fileparts(featureFiles(j).name);
                feature.cate(ca).image(im).name = name_;
                feature.cate(ca).image(im).path = fullfile(featurePath, categoryName, featureFiles(j).name);
% If needed, loading the feature vectors.
                if (loadVector)
                    vector = load(feature.cate(ca).image(im).path);
                    feature.cate(ca).image(im).vector = vector.features;
                end
            end
            feature.images = feature.images + feature.cate(ca).images;
            fprintf('Features of category %d are successfully loaded. %d images in total.\n', ...
                feature.categories, feature.images);
        end
    end
% Calculating the dimension of loaded features.
    vector = load(feature.cate(1).image(1).path);
    feature.dimensions = length(vector.features);
    fprintf('Features are successfully loaded. %d categories and %d images in total.\n', ...
        feature.categories, feature.images);
    finished = true;
else
    fprintf('Bad feature directory. Please check if %s exists.\n', featurePath);
    finished = false;
end

end
